--
-- upgrade_beta1_beta12.sql
-- 
-- Upgrade einer WASKA beta1 Datenbank auf WASKA beta12:
-- - CREATE FUNCTION waska_revoke_connect()
-- - REPLACE FUNCTION waska_drop_role()
BEGIN;

-- CREATE PROCEDURAL LANGUAGE plpgsql;

SET search_path = public, pg_catalog;

--
-- isEditorOrStandin(integer)
-- Liefert TRUE wenn der Benutzer fr die Fallakte 
-- a) Bearbeiter oder
-- b) Vertreter ist
-- Sonst FALSE
--
CREATE OR REPLACE FUNCTION isEditorOrStandin(mid integer)
RETURNS bool 
    LANGUAGE 'plpgsql' STABLE
    AS $$
DECLARE
bid integer;
isAllowed integer;
BEGIN
SELECT bearbeiter_id into bid from master_tbl_view WHERE id = mid;
IF bid = getuserid() THEN
RETURN TRUE;
ELSE 
SELECT INTO isAllowed benutzer_id FROM 
	nm_benutzer_master_tbl_view WHERE 
	master_id = mid AND benutzer_id = bid;
END IF;
RETURN FOUND;
END;
$$;

REVOKE ALL ON FUNCTION isEditorOrStandin( integer ) FROM public;
GRANT EXECUTE ON FUNCTION isEditorOrStandin(integer) TO GROUP :cm_ka_group;
GRANT EXECUTE ON FUNCTION isEditorOrStandin(integer) TO GROUP :admin_ka_group;
ALTER FUNCTION isEditorOrStandin( integer ) OWNER TO :db_adm;

-- waska_revoke_connect( nutzer, agentur ) returns boolean
-- Zurckziehen der CONNECT-Berechtigung des Nutzers fr die DB der Agentur.
CREATE OR REPLACE FUNCTION waska_revoke_connect (
        username varchar(64),
        agency varchar(52)
  ) RETURNS boolean LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
  AS $$
  DECLARE 
    db_name  varchar(64);
    db_user  varchar(128);
  BEGIN
    -- IF position(quote_literal(agency) in session_user) != 4
    IF position(agency in session_user) != 4
    THEN
      RAISE EXCEPTION 
          'Permission denied to drop role % in agency % by %.',
          quote_literal(username), quote_literal(agency), session_user;
    ELSE
      db_name := quote_ident('ka_' || agency || '_db');
      db_user := quote_ident('ka_' || agency || '_' || username);
      EXECUTE 'REVOKE CONNECT ON DATABASE ' || db_name
          || ' FROM ' || db_user ;
      RETURN true;
    END IF;
    RETURN false;
  END;
$$ ;
REVOKE ALL ON FUNCTION waska_revoke_connect( varchar, varchar ) FROM public;
GRANT EXECUTE ON FUNCTION waska_revoke_connect( varchar, varchar ) TO waska_role_adm;
GRANT EXECUTE ON FUNCTION waska_revoke_connect( varchar, varchar ) TO :admin_ka_group;
ALTER FUNCTION waska_revoke_connect(varchar, varchar) OWNER TO :db_adm;

-- waska_drop_role( nutzer, agentur ) returns boolean 
CREATE OR REPLACE FUNCTION waska_drop_role (
        username varchar(64),
        agency varchar(52)
  ) RETURNS boolean LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
  AS $$
  DECLARE 
    db_user       varchar(128);
    status_revoke boolean;
  BEGIN
    -- IF position(quote_literal(agency) in session_user) != 4
    IF position(agency in session_user) != 4
    THEN
      RAISE EXCEPTION 
          'Permission denied to drop role % in agency % by %.',
          quote_literal(username), quote_literal(agency), session_user;
    ELSE
      db_user := quote_ident('ka_' || agency || '_' || username);
      SELECT waska_revoke_connect(username, agency) INTO status_revoke;
      IF status_revoke
      THEN
        EXECUTE 'DROP ROLE ' || db_user ;
        RETURN true;
      END IF;
    END IF;
    RETURN false;
  END;
$$ ;
REVOKE ALL ON FUNCTION waska_drop_role(varchar, varchar) FROM public;

CREATE PROCEDURAL LANGUAGE 'plpythonu' HANDLER plpython_call_handler;
drop function waska_passgen(int);
-- Passwortgenerator variabler Lnge. Nutzt pwgen, um konforme Passworte zu
-- generieren
CREATE OR REPLACE FUNCTION waska_passgen(pw_length int) RETURNS text 
  LANGUAGE 'plpythonu' VOLATILE
  AS $$
    from os import popen
    from random import sample 
    pw = None
    try:
      try:
        pw = popen("/usr/bin/pwgen --ambiguous --numerals --symbols %s"
                    % pw_length)
        return pw.readline().strip()
      except:
      	pop = [chr(c) for c in range(33,127+1)]
        return "".join(sample(pop, pw_length)) 
    finally:
      if pw:
        try: pw.close()
        except: pass
$$;
ALTER FUNCTION waska_passgen(int) OWNER TO waska_role_adm;
REVOKE ALL ON FUNCTION waska_passgen(int) FROM public;


-- Funktionen zum Anlegen neuer (Teile von) Fallakten
-- Diese sind nur fr die Gruppe CM_KA aufrufbar (Admin_KA darf keine neuen 
-- Akten anlegen) und laufen mit den Berechtigungen des zentralen
-- administrativen Benutzers der KA
CREATE OR REPLACE FUNCTION create_master_ds() RETURNS integer
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
DECLARE
    standin integer;
BEGIN
INSERT INTO master_tbl (bearbeiter_id) VALUES (getuserid());
SELECT INTO standin vertreter from ka_benutzer_tbl WHERE id = getuserid();
IF standin IS NOT NULL THEN 
    INSERT INTO nm_benutzer_master_tbl(benutzer_id, master_id) VALUES (standin, 
currval('master_tbl_id_seq'));
END IF;
RETURN currval('master_tbl_id_seq');
END;
$$ ;

CREATE OR REPLACE FUNCTION ka_delete_role (
        loginname       varchar(64),
        agency          varchar(52)
  ) RETURNS VOID 
  LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
  AS $$
  DECLARE
    db_user  varchar(128);
  BEGIN
    db_user := quote_ident('ka_' || agency || '_' || loginname);
    DELETE FROM ka_benutzer_tbl WHERE 'login' = db_user;
    PERFORM waska_drop_role(loginname, agency);
  END;
$$ ;

ALTER FUNCTION ka_delete_role(varchar, varchar) OWNER TO :db_adm;
GRANT EXECUTE ON FUNCTION ka_delete_role(varchar, varchar) TO GROUP :admin_ka_group;
REVOKE ALL ON FUNCTION ka_delete_role(varchar, varchar) FROM public;

--
-- Untersttzungsangebote im Bereich der Berufsvorbereitung
--
CREATE OR REPLACE FUNCTION create_rg_beruf_ds(mid integer) RETURNS integer
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
BEGIN
IF NOT isEditorOrStandin(mid) THEN
RAISE EXCEPTION 'DS ID not session_user ID';
END IF;
INSERT INTO rg_angebote_berufsvorbereitung_tbl(master_id) VALUES (mid);
RETURN currval('rg_angebote_berufsvorbereitung_tbl_id_seq');

END;
$$ ;

GRANT EXECUTE ON FUNCTION create_rg_beruf_ds( integer ) TO GROUP :cm_ka_group;
REVOKE ALL ON FUNCTION create_rg_beruf_ds( integer ) FROM public;

CREATE OR REPLACE FUNCTION delete_rg_beruf_ds(dsid integer) RETURNS void 
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
DECLARE
mid integer;
BEGIN
SELECT INTO mid master_id FROM rg_angebote_berufsvorbereitung_tbl WHERE id = dsid;
IF NOT isEditorOrStandin(mid) THEN
RAISE EXCEPTION 'DS ID not session_user ID';
END IF;
DELETE FROM rg_angebote_berufsvorbereitung_tbl WHERE id = dsid;
END;
$$ ;

GRANT EXECUTE ON FUNCTION delete_rg_beruf_ds( integer ) TO GROUP :cm_ka_group;
REVOKE ALL ON FUNCTION delete_rg_beruf_ds( integer ) FROM public;


--
-- Untersttzungsangebote im allgemein bildenden Bereich
--
CREATE OR REPLACE FUNCTION create_rg_bilden_ds(mid integer) RETURNS integer
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
BEGIN
IF NOT isEditorOrStandin(mid) THEN
RAISE EXCEPTION 'DS ID not session_user ID';
END IF;
INSERT INTO rg_angebote_bildenden_bereich_tbl(master_id) VALUES (mid);
RETURN currval('rg_angebote_bildenden_bereich_tbl_id_seq');

END;
$$ ;

GRANT EXECUTE ON FUNCTION create_rg_bilden_ds( integer ) TO GROUP :cm_ka_group;
REVOKE ALL ON FUNCTION create_rg_bilden_ds( integer ) FROM public;

CREATE OR REPLACE FUNCTION delete_rg_bilden_ds(dsid integer) RETURNS void 
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
DECLARE
mid integer;
BEGIN
SELECT INTO mid master_id FROM rg_angebote_bildenden_bereich_tbl WHERE id = dsid;
IF not isEditorOrStandin(mid) THEN
RAISE EXCEPTION 'DS ID not session_user ID';
END IF;
DELETE FROM rg_angebote_bildenden_bereich_tbl WHERE id = dsid;
END;
$$ ;

GRANT EXECUTE ON FUNCTION delete_rg_bilden_ds( integer ) TO GROUP :cm_ka_group;
REVOKE ALL ON FUNCTION delete_rg_bilden_ds( integer ) FROM public; 


--
-- Untersttzungsangebote im Bereich der Lebensbewltigung
--
CREATE OR REPLACE FUNCTION create_rg_leben_ds(mid integer) RETURNS integer
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
BEGIN
IF NOT isEditorOrStandin(mid) THEN
RAISE EXCEPTION 'DS ID not session_user ID';
END IF;
INSERT INTO rg_angebote_lebensbewaeltigung_tbl(master_id) VALUES (mid);
RETURN currval('rg_angebote_lebensbewaeltigung_tbl_id_seq');

END;
$$ ;

GRANT EXECUTE ON FUNCTION create_rg_leben_ds( integer ) TO GROUP :cm_ka_group;
REVOKE ALL ON FUNCTION create_rg_leben_ds( integer ) FROM public;

CREATE OR REPLACE FUNCTION delete_rg_leben_ds(dsid integer) RETURNS void 
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
DECLARE
mid integer;
BEGIN
SELECT INTO mid master_id FROM rg_angebote_lebensbewaeltigung_tbl WHERE id = dsid;
IF NOT isEditorOrStandin(mid) THEN
RAISE EXCEPTION 'DS ID not session_user ID';
END IF;
DELETE FROM rg_angebote_lebensbewaeltigung_tbl WHERE id = dsid;
END;
$$ ;

GRANT EXECUTE ON FUNCTION delete_rg_leben_ds( integer ) TO GROUP :cm_ka_group;
REVOKE ALL ON FUNCTION delete_rg_leben_ds( integer ) FROM public;


--
-- Untersttzungsangebote im Bereich der beruflichen Qualifizierung
--
CREATE OR REPLACE FUNCTION create_rg_quali_ds(mid integer) RETURNS integer
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
BEGIN
IF NOT isEditorOrStandin(mid) THEN
RAISE EXCEPTION 'DS ID not session_user ID';
END IF;
INSERT INTO rg_angebote_berufliche_qualifizierung_tbl (master_id) VALUES (mid);
RETURN currval('rg_angebote_berufliche_qualifizierung_tbl_id_seq');

END;
$$ ;

GRANT EXECUTE ON FUNCTION create_rg_quali_ds( integer ) TO GROUP :cm_ka_group;
REVOKE ALL ON FUNCTION create_rg_quali_ds( integer ) FROM public;

CREATE OR REPLACE FUNCTION delete_rg_quali_ds(dsid integer) RETURNS void 
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
DECLARE
mid integer;
BEGIN
SELECT INTO mid master_id FROM rg_angebote_berufliche_qualifizierung_tbl WHERE id = dsid;
IF NOT isEditorOrStandin(mid) THEN
RAISE EXCEPTION 'DS ID not session_user ID';
END IF;
DELETE FROM rg_angebote_berufliche_qualifizierung_tbl WHERE id = dsid;
END;
$$ ;

GRANT EXECUTE ON FUNCTION delete_rg_quali_ds( integer ) TO GROUP :cm_ka_group;
REVOKE ALL ON FUNCTION delete_rg_quali_ds( integer ) FROM public;


--
-- Massnahmen 
--
CREATE OR REPLACE FUNCTION create_rg_massnahme_ds(mid integer) RETURNS integer
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
BEGIN
IF NOT isEditorOrStandin(mid) THEN
RAISE EXCEPTION 'DS ID not session_user ID';
END IF;
INSERT INTO rg_massnahme_tbl(master_id) VALUES (mid);
RETURN currval('rg_massnahme_tbl_id_seq');

END;
$$ ;

GRANT EXECUTE ON FUNCTION create_rg_massnahme_ds( integer ) TO GROUP :cm_ka_group;
REVOKE ALL ON FUNCTION create_rg_massnahme_ds( integer ) FROM public;


--
-- Kompetenzfeststellung
--
CREATE OR REPLACE FUNCTION create_rg_kompetenz_ds(mid integer) RETURNS integer
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
BEGIN
IF NOT isEditorOrStandin(mid) THEN
RAISE EXCEPTION 'DS ID not session_user ID';
END IF;
INSERT INTO rg_kompetenzfestellung_tbl(master_id) VALUES (mid);
RETURN currval('rg_kompetenzfestellung_tbl_id_seq');

END;
$$ ;

GRANT EXECUTE ON FUNCTION create_rg_kompetenz_ds( integer ) TO GROUP :cm_ka_group;
REVOKE ALL ON FUNCTION create_rg_kompetenz_ds( integer ) FROM public; 


--
-- Sprachen
--
CREATE OR REPLACE FUNCTION create_rg_sprache_ds(rgid integer) RETURNS integer
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
DECLARE
        mid integer;
BEGIN
	SELECT INTO mid master_id FROM rg_kompetenzfestellung_tbl WHERE id = rgid;
        IF NOT isEditorOrStandin(mid) THEN                 
		RAISE EXCEPTION 'DS ID not session_user ID';         
	END IF;         
	INSERT INTO rg_beherrschung_sprache_tbl(rg_kompetenzfestellung_id) 
	VALUES (rgid);
        RETURN currval('rg_beherrschung_sprache_tbl_id_seq');

END;
$$ ;

GRANT EXECUTE ON FUNCTION create_rg_sprache_ds( integer ) TO GROUP :cm_ka_group;
REVOKE ALL ON FUNCTION create_rg_sprache_ds( integer ) FROM public;


--
-- Funktionen zum erstellen von Dokumenten
--

CREATE OR REPLACE FUNCTION create_fall_dokument_ds(mid integer) RETURNS integer
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
BEGIN
	IF NOT isEditorOrStandin(mid) THEN
	RAISE EXCEPTION 'DS ID not session_user ID';
	END IF;
	INSERT INTO ka_dokumente_tbl(master_id) VALUES (mid);
	RETURN currval('ka_dokumente_tbl_id_seq');
END;
$$ ;

CREATE OR REPLACE FUNCTION delete_fall_dokument_ds(did integer) RETURNS void 
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
DECLARE
        mid integer;
BEGIN
	SELECT INTO mid master_id FROM ka_dokumente_tbl WHERE id = did;
	IF NOT isEditorOrStandin(mid) THEN
	RAISE EXCEPTION 'DS ID not session_user ID';
	END IF;
	DELETE FROM ka_dokumente_tbl WHERE id = did;
END;
$$ ;

GRANT EXECUTE ON FUNCTION create_fall_dokument_ds( integer ) TO GROUP :cm_ka_group;
GRANT EXECUTE ON FUNCTION delete_fall_dokument_ds( integer ) TO GROUP :cm_ka_group;
REVOKE ALL ON FUNCTION create_fall_dokument_ds( integer ) FROM public;
REVOKE ALL ON FUNCTION delete_fall_dokument_ds( integer ) FROM public;

-- Changes in the db views (splitting of "kompetenzfestellung")
DROP VIEW page_9_view;

CREATE OR REPLACE VIEW page_9_view AS SELECT
    rg_kompetenzfestellung_tbl.id AS id,
    rg_kompetenzfestellung_tbl.rechenfaehigkeit AS rechenfaehigkeit,
    rg_kompetenzfestellung_tbl.raeumliches_vorstellungsvermoegen AS raeumliches_vorstellungsvermoegen,
    rg_kompetenzfestellung_tbl.grundverstaendnis_naturwissenschaftlicher_zusammenhaenge AS grundverstaendnis_naturwissenschaftlicher_zusammenhaenge,
    rg_kompetenzfestellung_tbl.logisches_denken AS logisches_denken,
    rg_kompetenzfestellung_tbl.wirtschaftliche_grundkenntnisse AS wirtschaftliche_grundkenntnisse,
    rg_kompetenzfestellung_tbl.allgemeine_pc_kenntnisse AS allgemeine_pc_kenntnisse,
    rg_kompetenzfestellung_tbl.spezielle_pc_kenntnisse_grafikbearbeitung_musikbearbeitung AS spezielle_pc_kenntnisse_grafikbearbeitung_musikbearbeitung,
    rg_kompetenzfestellung_tbl.internet_anwendungen_und_e_mail_kenntnisse AS internet_anwendungen_und_e_mail_kenntnisse,
    rg_kompetenzfestellung_tbl.andere_medien AS andere_medien,
    rg_kompetenzfestellung_tbl.hand_fingergeschick AS hand_fingergeschick,
    rg_kompetenzfestellung_tbl.formauffassung AS formauffassung,
    rg_kompetenzfestellung_tbl.umgang_maschinen_werkzeugen AS umgang_maschinen_werkzeugen,
    rg_kompetenzfestellung_tbl.gestalterische_fantasie AS gestalterische_fantasie
FROM rg_kompetenzfestellung_tbl
    INNER JOIN master_tbl
    ON         rg_kompetenzfestellung_tbl.master_id = master_tbl.id
WHERE
    master_tbl.bearbeiter_id = getuserid()
    OR master_tbl.id IN (
        SELECT nm.master_id from nm_benutzer_master_tbl nm
            WHERE nm.benutzer_id = getuserid())
    OR pg_has_role('admin_ka','MEMBER');

ALTER TABLE page_9_view OWNER TO :db_adm;

CREATE OR REPLACE RULE page_9_update_rule AS ON UPDATE TO page_9_view
DO INSTEAD UPDATE rg_kompetenzfestellung_tbl SET
    rechenfaehigkeit = NEW.rechenfaehigkeit,
    raeumliches_vorstellungsvermoegen = NEW.raeumliches_vorstellungsvermoegen,
    grundverstaendnis_naturwissenschaftlicher_zusammenhaenge = NEW.grundverstaendnis_naturwissenschaftlicher_zusammenhaenge,
    logisches_denken = NEW.logisches_denken,
    wirtschaftliche_grundkenntnisse = NEW.wirtschaftliche_grundkenntnisse,
    allgemeine_pc_kenntnisse = NEW.allgemeine_pc_kenntnisse,
    spezielle_pc_kenntnisse_grafikbearbeitung_musikbearbeitung = NEW.spezielle_pc_kenntnisse_grafikbearbeitung_musikbearbeitung,
    internet_anwendungen_und_e_mail_kenntnisse = NEW.internet_anwendungen_und_e_mail_kenntnisse,
    andere_medien = NEW.andere_medien,
    hand_fingergeschick = NEW.hand_fingergeschick,
    formauffassung = NEW.formauffassung,
    umgang_maschinen_werkzeugen = NEW.umgang_maschinen_werkzeugen,
    gestalterische_fantasie = NEW.gestalterische_fantasie
WHERE id = NEW.id;

GRANT SELECT ON page_9_view TO GROUP :cm_ka_group;
GRANT UPDATE ON page_9_view TO GROUP :cm_ka_group;
GRANT SELECT ON page_9_view TO GROUP :admin_ka_group;

CREATE OR REPLACE RULE page_9_delete_rule AS ON DELETE TO page_9_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE page_9_insert_rule AS ON INSERT TO page_9_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE VIEW page_26_view AS SELECT
    rg_kompetenzfestellung_tbl.id AS id,
    rg_kompetenzfestellung_tbl.methoden_der_informationsgewinnung AS methoden_der_informationsgewinnung,
    rg_kompetenzfestellung_tbl.methoden_der_informationsverarbeitung AS methoden_der_informationsverarbeitung,
    rg_kompetenzfestellung_tbl.einsatz_lern_und_arbeitstechniken AS einsatz_lern_und_arbeitstechniken,
    rg_kompetenzfestellung_tbl.problemloesungsorientierung AS problemloesungsorientierung
FROM rg_kompetenzfestellung_tbl
    INNER JOIN master_tbl
    ON         rg_kompetenzfestellung_tbl.master_id = master_tbl.id
WHERE
    master_tbl.bearbeiter_id = getuserid()
    OR master_tbl.id IN (
        SELECT nm.master_id from nm_benutzer_master_tbl nm
            WHERE nm.benutzer_id = getuserid())
    OR pg_has_role('admin_ka','MEMBER');

ALTER TABLE page_26_view OWNER TO :db_adm;

CREATE OR REPLACE RULE page_26_update_rule AS ON UPDATE TO page_26_view
DO INSTEAD UPDATE rg_kompetenzfestellung_tbl SET
    methoden_der_informationsgewinnung = NEW.methoden_der_informationsgewinnung,
    methoden_der_informationsverarbeitung = NEW.methoden_der_informationsverarbeitung,
    einsatz_lern_und_arbeitstechniken = NEW.einsatz_lern_und_arbeitstechniken,
    problemloesungsorientierung = NEW.problemloesungsorientierung
WHERE id = NEW.id;

GRANT SELECT ON page_26_view TO GROUP :cm_ka_group;
GRANT UPDATE ON page_26_view TO GROUP :cm_ka_group;
GRANT SELECT ON page_26_view TO GROUP :admin_ka_group;

CREATE OR REPLACE RULE page_26_delete_rule AS ON DELETE TO page_26_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE page_26_insert_rule AS ON INSERT TO page_26_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE VIEW page_27_view AS SELECT
    rg_kompetenzfestellung_tbl.id AS id,
    rg_kompetenzfestellung_tbl.texte_lesen_und_verstehen AS texte_lesen_und_verstehen,
    rg_kompetenzfestellung_tbl.texte_schreiben_und_formulieren AS texte_schreiben_und_formulieren,
    rg_kompetenzfestellung_tbl.deutsche_sprache AS deutsche_sprache,
    rg_kompetenzfestellung_tbl.sprache1 AS sprache1,
    rg_kompetenzfestellung_tbl.sprache1_sonstige_nennung AS sprache1_sonstige_nennung,
    rg_kompetenzfestellung_tbl.beherrschung_sprache1 AS beherrschung_sprache1,
    rg_kompetenzfestellung_tbl.sprache2 AS sprache2,
    rg_kompetenzfestellung_tbl.sprache2_sonstige_nennung AS sprache2_sonstige_nennung,
    rg_kompetenzfestellung_tbl.beherrschung_sprache2 AS beherrschung_sprache2,
    rg_kompetenzfestellung_tbl.sprache3 AS sprache3,
    rg_kompetenzfestellung_tbl.sprache3_sonstige_nennung AS sprache3_sonstige_nennung,
    rg_kompetenzfestellung_tbl.beherrschung_sprache3 AS beherrschung_sprache3
FROM rg_kompetenzfestellung_tbl
    INNER JOIN master_tbl
    ON         rg_kompetenzfestellung_tbl.master_id = master_tbl.id
WHERE
    master_tbl.bearbeiter_id = getuserid()
    OR master_tbl.id IN (
        SELECT nm.master_id from nm_benutzer_master_tbl nm
            WHERE nm.benutzer_id = getuserid())
    OR pg_has_role('admin_ka','MEMBER');

ALTER TABLE page_27_view OWNER TO :db_adm;

CREATE OR REPLACE RULE page_27_update_rule AS ON UPDATE TO page_27_view
DO INSTEAD UPDATE rg_kompetenzfestellung_tbl SET
    texte_lesen_und_verstehen = NEW.texte_lesen_und_verstehen,
    texte_schreiben_und_formulieren = NEW.texte_schreiben_und_formulieren,
    deutsche_sprache = NEW.deutsche_sprache,
    sprache1 = NEW.sprache1,
    sprache1_sonstige_nennung = NEW.sprache1_sonstige_nennung,
    beherrschung_sprache1 = NEW.beherrschung_sprache1,
    sprache2 = NEW.sprache2,
    sprache2_sonstige_nennung = NEW.sprache2_sonstige_nennung,
    beherrschung_sprache2 = NEW.beherrschung_sprache2,
    sprache3 = NEW.sprache3,
    sprache3_sonstige_nennung = NEW.sprache3_sonstige_nennung,
    beherrschung_sprache3 = NEW.beherrschung_sprache3
WHERE id = NEW.id;

GRANT SELECT ON page_27_view TO GROUP :cm_ka_group;
GRANT UPDATE ON page_27_view TO GROUP :cm_ka_group;
GRANT SELECT ON page_27_view TO GROUP :admin_ka_group;

CREATE OR REPLACE RULE page_27_delete_rule AS ON DELETE TO page_27_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE page_27_insert_rule AS ON INSERT TO page_27_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE VIEW page_28_view AS SELECT
    rg_kompetenzfestellung_tbl.id AS id,
    rg_kompetenzfestellung_tbl.kommunikationsfaehigkeit AS kommunikationsfaehigkeit,
    rg_kompetenzfestellung_tbl.kritikfaehigkeit AS kritikfaehigkeit,
    rg_kompetenzfestellung_tbl.konfliktfaehigkeit AS konfliktfaehigkeit,
    rg_kompetenzfestellung_tbl.kreativitaet AS kreativitaet,
    rg_kompetenzfestellung_tbl.team_und_kooperationsfaehigkeit AS team_und_kooperationsfaehigkeit,
    rg_kompetenzfestellung_tbl.solidarisches_verhalten AS solidarisches_verhalten,
    rg_kompetenzfestellung_tbl.selbstaendiges_lernen_arbeiten AS selbstaendiges_lernen_arbeiten,
    rg_kompetenzfestellung_tbl.selbstreflexion_selbsteinschaetzung AS selbstreflexion_selbsteinschaetzung,
    rg_kompetenzfestellung_tbl.verantwortungsuebernahme AS verantwortungsuebernahme,
    rg_kompetenzfestellung_tbl.zielorientierung AS zielorientierung,
    rg_kompetenzfestellung_tbl.zuverlaessigkeit AS zuverlaessigkeit,
    rg_kompetenzfestellung_tbl.altersgerechter_entwicklungsstand AS altersgerechter_entwicklungsstand,
    rg_kompetenzfestellung_tbl.gesundheitliche_voraussetzungen AS gesundheitliche_voraussetzungen,
    rg_kompetenzfestellung_tbl.konzentration AS konzentration,
    rg_kompetenzfestellung_tbl.ausdauer AS ausdauer,
    rg_kompetenzfestellung_tbl.durchhaltevermoegen AS durchhaltevermoegen,
    rg_kompetenzfestellung_tbl.merkfaehigkeit AS merkfaehigkeit,
    rg_kompetenzfestellung_tbl.bearbeitungsgeschwindigkeit AS bearbeitungsgeschwindigkeit,
    rg_kompetenzfestellung_tbl.befaehigung_zu_daueraufmerksamkeit AS befaehigung_zu_daueraufmerksamkeit,
    rg_kompetenzfestellung_tbl.leistungsbereitschaft AS leistungsbereitschaft,
    rg_kompetenzfestellung_tbl.sorgfalt AS sorgfalt,
    rg_kompetenzfestellung_tbl.umgangsformen AS umgangsformen
FROM rg_kompetenzfestellung_tbl
    INNER JOIN master_tbl
    ON         rg_kompetenzfestellung_tbl.master_id = master_tbl.id
WHERE
    master_tbl.bearbeiter_id = getuserid()
    OR master_tbl.id IN (
        SELECT nm.master_id from nm_benutzer_master_tbl nm
            WHERE nm.benutzer_id = getuserid())
    OR pg_has_role('admin_ka','MEMBER');

ALTER TABLE page_28_view OWNER TO :db_adm;

CREATE OR REPLACE RULE page_28_update_rule AS ON UPDATE TO page_28_view
DO INSTEAD UPDATE rg_kompetenzfestellung_tbl SET
    kommunikationsfaehigkeit = NEW.kommunikationsfaehigkeit,
    kritikfaehigkeit = NEW.kritikfaehigkeit,
    konfliktfaehigkeit = NEW.konfliktfaehigkeit,
    kreativitaet = NEW.kreativitaet,
    team_und_kooperationsfaehigkeit = NEW.team_und_kooperationsfaehigkeit,
    solidarisches_verhalten = NEW.solidarisches_verhalten,
    selbstaendiges_lernen_arbeiten = NEW.selbstaendiges_lernen_arbeiten,
    selbstreflexion_selbsteinschaetzung = NEW.selbstreflexion_selbsteinschaetzung,
    verantwortungsuebernahme = NEW.verantwortungsuebernahme,
    zielorientierung = NEW.zielorientierung,
    zuverlaessigkeit = NEW.zuverlaessigkeit,
    altersgerechter_entwicklungsstand = NEW.altersgerechter_entwicklungsstand,
    gesundheitliche_voraussetzungen = NEW.gesundheitliche_voraussetzungen,
    konzentration = NEW.konzentration,
    ausdauer = NEW.ausdauer,
    durchhaltevermoegen = NEW.durchhaltevermoegen,
    merkfaehigkeit = NEW.merkfaehigkeit,
    bearbeitungsgeschwindigkeit = NEW.bearbeitungsgeschwindigkeit,
    befaehigung_zu_daueraufmerksamkeit = NEW.befaehigung_zu_daueraufmerksamkeit,
    leistungsbereitschaft = NEW.leistungsbereitschaft,
    sorgfalt = NEW.sorgfalt,
    umgangsformen = NEW.umgangsformen
WHERE id = NEW.id;

GRANT SELECT ON page_28_view TO GROUP :cm_ka_group;
GRANT UPDATE ON page_28_view TO GROUP :cm_ka_group;
GRANT SELECT ON page_28_view TO GROUP :admin_ka_group;

CREATE OR REPLACE RULE page_28_delete_rule AS ON DELETE TO page_28_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE page_28_insert_rule AS ON INSERT TO page_28_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE VIEW page_29_view AS SELECT
    rg_kompetenzfestellung_tbl.id AS id,
    rg_kompetenzfestellung_tbl.dokumentation_kompetenzfeststellung AS dokumentation_kompetenzfeststellung
FROM rg_kompetenzfestellung_tbl
    INNER JOIN master_tbl
    ON         rg_kompetenzfestellung_tbl.master_id = master_tbl.id
WHERE
    master_tbl.bearbeiter_id = getuserid()
    OR master_tbl.id IN (
        SELECT nm.master_id from nm_benutzer_master_tbl nm
            WHERE nm.benutzer_id = getuserid())
    OR pg_has_role('admin_ka','MEMBER');

ALTER TABLE page_29_view OWNER TO :db_adm;

CREATE OR REPLACE RULE page_29_update_rule AS ON UPDATE TO page_29_view
DO INSTEAD UPDATE rg_kompetenzfestellung_tbl SET
    dokumentation_kompetenzfeststellung = NEW.dokumentation_kompetenzfeststellung
WHERE id = NEW.id;

GRANT SELECT ON page_29_view TO GROUP :cm_ka_group;
GRANT UPDATE ON page_29_view TO GROUP :cm_ka_group;
GRANT SELECT ON page_29_view TO GROUP :admin_ka_group;

CREATE OR REPLACE RULE page_29_delete_rule AS ON DELETE TO page_29_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE page_29_insert_rule AS ON INSERT TO page_29_view
    DO INSTEAD NOTHING;

COMMIT;
