# -*- coding: utf-8 -*-
#
# Copyright 2007, 2008 Intevation GmbH, Germany, <info@intevation.de>
# 
# This file is part of mpuls WASKA (CoMPUter-based case fiLeS - 
# Web-Anwendungs-Server fuer Kompetenzagenturen).
# 
# mpuls WASKA is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
# 
# mpuls WASKA is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
# 
# You should have received a copy of the GNU Affero General Public
# License along with mpuls WASKA. If not, see <http://www.gnu.org/licenses/>.
# 
# mpuls WASKA has been developed on behalf of the 
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and 
# European Social Fund resources.
#
# Authors:
# Sascha L. Teichmann <teichmann@intevation.de>
#

from waskaweb.model.nodecomponents import Node
import waskaweb.lib.helpers as h
import waskaweb.lib.filters as F


import waskaweb.model.data as data

from cgi import escape

import re
from datetime import date 

TARGET=re.compile(r"(\w+)(?:|:(.+))$")

HELP = \
'''<span class="help"> <a href="%s" target="_blank">
<img src="/images/icons/form_help.png" border="0"
alt="Hilfesymbol"></a></span>'''

REQUIRED = \
'''<span class="help"> <a href="%s" target="_blank">
<img src="/images/icons/required_12x12.png" border="0" alt="Pflichtfeld"></a></span>'''

EVALUATION = \
'''<span class="help"> <a href="%s" target="_blank">
<img src="/images/icons/needed_12x12.png" border="0" alt="Auswertungsrelevant"></a></span>'''

WEIGHTS = (15, 25, 30, 40, 45, 50, 55, 60, 75)

SET_MODIFICATION = ' onchange="setModification();" '

class FormData:
   
    def __init__(self, help=None):
        self.help = help 

    def getData(self, dataID):
        return ""

    def getPrintableData(self, dataID):
        return self.getData(dataID) 

    def getHelp(self, dataID):
        if not self.help: return ''

        out = []

        if self.help.hasAnnotations(dataID):
            out.append(REQUIRED % h.url_for('required', help_id=escape(dataID)))

        if self.help.hasAnnotations(dataID, ('evaluation',)):
            out.append(EVALUATION % h.url_for('showevalhelp', help_id=escape(dataID)))

        if self.help.hasHelp(dataID):
            out.append(HELP % h.url_for('formhelp', help_id=escape(dataID)))

        return ''.join(out)

    def getError(self, dataID):
        return ""

def digits(n):
    d, c = 10, 1
    while n >= d: d *= 10; c += 1
    return c

def visibleDepth(c):
    depth = -1
    while c:
        if not(isinstance(c, data.GroupNode) and not c.isInvisible()):
            depth += 1
        c = c.parent
    return depth

def pretty(x):
    if isinstance(x, date):
        return "%02d.%02d.%04d" % (x.day, x.month, x.year)
    if isinstance(x, (type(""), type(u""))):
        return u"%s" % x
        #return unicode(str(x), 'utf-8')
    if isinstance(x, type(False)):
        return x and "on" or ""
    return str(x)

class ErrorItem:

    def __init__(self, page=None, bad=None, name=None):
        self.page     = page
        self.bad      = bad
        self.name     = name
        self.messages = []

    def addMessage(self, msg):
        self.messages.append(msg)

    def getName(self):
        return self.name

    def getMessages(self):
        return self.messages


class FormDataImpl(FormData):

    def __init__(self, help=None, page=None, errors=None, nc=None):
        FormData.__init__(self, help)
        self.page   = page
        self.errors = errors
        if nc:
            self.nc     = nc #NodeComponent List []
            self.widgets = self.nc[0].allWidgets()
        else: 
            self.nc = []
            self.widgets = {} 

    def getData(self, dataID):
        if not self.errors is None:
            try:
                ei = self.errors[dataID]
                if not ei.bad is None:
                    return u"%s" % pretty(ei.bad)
            except KeyError:
                pass
        if not self.page is None:
            value = self.page.getData(dataID)
            if value is not None: value = pretty(value)
            else:     value = ""
            return value
        return ""

    def getPrintableData(self, dataID):
        value = self.getData(dataID)
        try:
            widget = self.widgets[dataID]
        except KeyError:
            return value 

        if not isinstance(widget, ChoiceNode):
            return value

        for c in widget.children:
            if isinstance(c, BoolLeaf):
                v = c.getValue()
                if v == value:
                    return c.getDescription()
            elif isinstance(c, ExternalChoiceListLeaf):
                cc = c.getChildren()
                if cc:
                    for i in cc:
                        if isinstance(i, BoolLeaf):
                            v = c.getValue()
                            if v == value:
                                return c.getDescription()

        return value 

    def getError(self, dataID):
        if self.errors:
            try:
                ei = self.errors[dataID]
                return u"<br>\n".join([u"%s" % s for s in ei.messages])
            except KeyError:
                pass
        return ""


class Text:
    def __init__(self, txt):
        self.txt = txt

    def render(self, target, weight):
        return self.txt

class Div(Text):
    def __init__(self, txt):
        Text.__init__(self, txt)

    def render(self, target, weight):
        return '<div class="waska_form_element w100">%s</div>\n' % self.txt

class Item(Text):
    def __init__(self, txt):
        Text.__init__(self, txt)

    def render(self, target, weight):
        weight = int(100.0*weight+0.5)
        weight = weight in WEIGHTS and " w%d" % weight or " w100"
        return '<div class="waska_form_element %s">\n%s</div>\n' % (
            weight, self.txt)

class Container:
    def __init__(self, target):
        self.target  = target
        self.items   = []

    def append(self, item, weight=1.0):
        self.items.append((item, weight))

    def render(self, doBreak=True):
        if self.items:
            out = []
            f = 1.0/reduce(lambda x, y: x + y[1], self.items, 0)
            ws = [f*w[1] for w in self.items]
            target = self.target
            out = [item[0].render(target, w) for item, w in zip(self.items, ws)]
            if doBreak: out.append('<br>')
            return "".join(out)
        return ""

class AidListRenderer:
    def __init__(self, aidList=None, ro_mode=True, no_data=False):
        self.aidList = aidList
        self.ro_mode = ro_mode
        self.no_data = no_data

    def render(self, ds_id, page_id):
        from waskaweb.lib.base import _ 

        action = self.ro_mode and "show" or "edit"
        # Translated string here
        s_inst     = _('cm_rg_aid_overview_table_institution')
        s_type     = _('cm_rg_aid_overview_table_offer_type')
        s_act      = _('cm_rg_aid_overview_table_actions')
        #s_show     = _('cm_rg_aid_overview_table_show')
        s_edit     = _('cm_rg_aid_overview_table_edit')
        s_goal     = _('cm_rg_aid_overview_table_goal')
        s_show     = '<img src="/images/icons/view_red.gif" border="0" alt="' + _('cm_overview_a_show') + \
            '" title="' + _('cm_overview_a_show') + '">'
        s_delete   = '<img src="/images/icons/delete_red.gif" border="0" alt="' + _('cm_overview_a_delete') + \
            '" title="' + _('cm_overview_a_delete') + '">'
        s_notfound = _('cm_rg_aid_overview_table_no_offer_found')
        s_legend   = _('cm_rg_aid_overview_legend_legend')
        s_bb       = _('cm_rg_aid_overview_legend_bb')
        s_bb_long  = _('cm_rg_aid_overview_legend_common_educational_area')
        s_bv       = _('cm_rg_aid_overview_legend_bv')
        s_bv_long  = _('cm_rg_aid_overview_legend_vocational_preparation')
        s_bq       = _('cm_rg_aid_overview_legend_bq')
        s_bq_long  = _('cm_rg_aid_overview_legend_qualifications')
        s_lb       = _('cm_rg_aid_overview_legend_lb')
        s_lb_long  = _('cm_rg_aid_overview_legend_life_skills')
        s_newoffer = _('cm_rg_aid_overview_legend_new_offer')

        out = []
        # Build table header
        out.append(u"""<div class="waska_form_element  w100"><table>
            <tr>
                <th class="table_header_h table_date">Von</th>
                <th class="table_header_h table_date">Bis</th>
                <th class="table_header_h">%s</th>
                <th class="table_header_h">Art der Unterstützung</th>
                <th class="table_header_h table_width_small">%s</th>
                <th class="table_header_h table_action">%s</th>
            </tr>""" % (s_inst, s_type, s_act))
        idset = True
        if not self.no_data:
            for aid in self.aidList.getDatasets():
                class_str = ""
                if idset:
                    out.append(u"""<tr><td>%s</td><td>%s</td><td>%s</td><td>%s</td><td>%s</td>""" % (F.NA(aid.start_date), F.NA(aid.end_date), F.NA(aid.institution), F.NA(aid.type), F.NA(aid.category)))
                    out.append(u"""<td class="table_action"><a href="/rg_aid/%s/%s/%s">%s</a>""" % (action,aid.id, aid.page_id, s_show))
                    if not self.ro_mode and h.hasRole(['cm_ka']):
                        out.append(u"""<a href="/rg_aid/delete/%s/%s/0">%s</a>""" % (aid.id, aid.category_id, s_delete))
                    out.append("""</td></tr>""")
                    out.append(u"""<tr><td colspan="6">%s: %s</td></tr>""" % (s_goal, F.NA(aid.goal)))
                else:
                    out.append(u"""<tr class="table_row_v"><td>%s</td><td>%s</td><td>%s</td><td>%s</td><td>%s</td>""" % (F.NA(aid.start_date), F.NA(aid.end_date), F.NA(aid.institution), F.NA(aid.type), F.NA(aid.category)))
                    out.append(u"""<td class="table_action"><a href="/rg_aid/%s/%s/%s">%s</a>""" % (action,aid.id, aid.page_id, s_show))
                    if not self.ro_mode and h.hasRole(['cm_ka']):
                        out.append(u"""<a href="/rg_aid/delete/%s/%s/0">%s</a>""" % (aid.id, aid.category_id, s_delete))
                    out.append("""</td></tr>""")
                    out.append(u"""<tr class="table_row_v"><td colspan="6">%s: %s</td>""" % (s_goal, F.NA(aid.goal)))
                idset = not idset
                # out.append(u"""<td class="table_action"><a href="/rg_aid/%s/%s/%s">%s</a>""" % (action,aid.id, aid.page_id, s_show))
                # if not self.ro_mode and h.hasRole(['cm_ka']):
                #     out.append(u"""<a href="/rg_aid/delete/%s/%s/0">%s</a>""" % (aid.id, aid.category_id, s_delete))
                # out.append("""</td></tr>""")

                if self.aidList.numDatasets() <= 0:
                    out.append(u"""<tr><td colspan="5">%s</td></tr>""" % (s_notfound))
        out.append(u"""</table>""")
        out.append(u"""<div class="legend">
        %s <strong>%s</strong>: %s <strong>%s</strong>: %s <strong>%s</strong>: %s <strong>%s</strong>: %s 
        </div></div>""" % (s_legend,s_bb, s_bb_long, s_bv, s_bv_long, s_bq, s_bq_long, s_lb, s_lb_long))

        if not self.ro_mode and h.hasRole(['cm_ka']):
            out.append(u"""<div class="waska_form">
  <form name="new_unterstuetzung" action="/rg_aid/newAction" method="POST">
  <label for="angebot">%s</label><br>""" % s_newoffer)
            out.append(u"""<select name="aid_type" id="angebot">
    <option value="0">%s</option>
    <option value="1">%s</option>
    <option value="2">%s</option>
    <option value="3">%s</option>
  </select>""" % (s_bb_long, s_bv_long, s_bq_long, s_lb_long))
            out.append(u"""<input type="hidden" name="page_id" value="%s">
  <input type="hidden" name="case_id" value="%s">
  <input type="submit" value="Hinzufügen">""" % (page_id, ds_id))
            out.append(u"""</form></div>""")
        return "\n".join(out)

class ViewRenderer:

    def __init__(self, formdata=FormData(), ro_mode=True):
        self.formdata        = formdata
        self.stateStack      = None
        self.containersStack = None
        self.ro_mode         = ro_mode
        self.items           = formdata.nc


    def renderView(self, name="", case_id="", action="", showNext=True):

        if not self.items:
            return "No items given!"
       
        out = []
        if not self.ro_mode:
            out.append('<form id="waska_form" action="')
            if action: out.append(escape(action, True))
            out.append('" accept-charset="UTF-8" method="POST">\n')
            # Add submitbutton at the top of the page too
            out.append('<div class="waska_form_element w100">')
            out.append('<input type="submit" name="__formular_save" value="Speichern">\n')
            if showNext:
                out.append('<input type="submit" name="__formular_next" value="Speichern und weiter">\n')
            out.append('</div>')

        self.stateStack = []
        self.containersStack = [[Container(None) ]]
        for item in self.items:
            self._renderChild(item)

        out.append(self.containersStack[-1][0].render())

        if not self.ro_mode:
            if action:
                if name:
                    out.append('<input type="hidden" name="page" value="%s">\n' \
                        % escape(name, True))
                if case_id:
                    out.append('<input type="hidden" name="ds" value="%s">\n' \
                        % escape(str(case_id), True))
                out.append('<input type="submit" name="__formular_save" value="Speichern">\n')
                if showNext:
                    out.append('<input type="submit" name="__formular_next" value="Speichern und weiter">\n')
            out.append('</form>\n')

        return "".join(out)

    def toTarget(self, item, target):
        weight = 1.0
        if target:
            m = TARGET.match(target)
            if m:
                target = m.group(1)
                try:
                   weight = float(m.group(2))
                except (TypeError, ValueError):
                   pass

        containersStack = self.containersStack
        for container in containersStack[-1]:
            if not container.target or container.target == target:
                container.append(item, weight)
                return

    def _renderGroup(self, node):
        description = node.getDescription()
        name        = node.getName()

        help        = self.formdata.getHelp(name)
        data        = self.formdata.getData(name)
        error       = self.formdata.getError(name)

        out = []
        
        error = self.formdata.getData(name)
        if node.isInvisible():
            if description:
                out.append("<!-- %s -->\n" % escape(description))
            if error:
                out.append('<!--<p class="errormsg">%s</p>-->' % escape(error))

            out.append(self._renderRecursive(node))
        else:
            out.append("<fieldset>\n")
            if description:
                out.append(
                    "<legend>%s %s</legend>\n" % (escape(description), help))
            if error:
                out.append('<!--<p class="errormsg">%s</p>-->\n' % error)
            out.append(self._renderRecursive(node))
            out.append("</fieldset>\n")

        self.toTarget(Item("".join(out)), node.getTarget())

    def _renderInfo(self, info):
        if info.isInvisible(): return
        text, depth = info.getValue(), visibleDepth(info)
        if text:
            text = "<h%s>%s</h%s>\n" % (depth, escape(text), depth)
            self.toTarget(Div(text), info.getTarget())

    def _renderRadio(self, radio):
        description = radio.getDescription()
        name        = radio.getName()

        help        = self.formdata.getHelp(name)
        data        = self.formdata.getData(name)
        error       = self.formdata.getError(name)

        label = self._renderLabel(description, name, help=help,)

        out = [ label ]

        self.stateStack.append(data)
        out.append(self._renderRecursive(radio))
        self.stateStack.pop()

        self.toTarget(Text("".join(out)), radio.getTarget());

    def _renderChoice(self, choice):
        description = choice.getDescription()
        name        = choice.getName()
        size        = choice.getSize()
        multiple    = choice.getMultiple()

        help        = self.formdata.getHelp(name)
        data        = self.formdata.getData(name)
        error       = self.formdata.getError(name)
        css_class   = []

        label = self._renderLabel(description, name, help=help)

        if name:
            name = escape(name, True)
            name = 'name="%s" id="%s"' % (name, name)
        else:
            name = ""

        # Do not render choicelists in disabled mode as IE can not cope with
        # css on disabled list elements. Instead just render the selected
        # option within the list (ChoiceBool)
        disabled = ""
        #if self.ro_mode: 
        #    disabled = 'disabled="disabled"'
        #    css_class.append('readonly')
        #else: disabled = ""

        multiple = multiple and "multiple" or ""

        if not size: size = "1"

        out = [ label ]

        if error:
            css_class.append('error_box')

        out.append(
            '<select size="%s" class="%s" %s %s %s %s>\n' % (
                size, " ".join(css_class), disabled, multiple, name, SET_MODIFICATION))

        self.stateStack.append(data)
        out.append(self._renderRecursive(choice, False))
        self.stateStack.pop()

        out.append("</select>\n")

        self.toTarget(Item("".join(out)), choice.getTarget());

    def _renderChoiceBool(self, bool):

        description = bool.getDescription()
        name        = bool.getName()
        value       = bool.getValue()
        selected    = ""
        out         = ""
        
        toSelect = self.stateStack[-1]
        selected = toSelect ==  (value or value==0) and "selected" or ""

        if (value or value==0): value = 'value="%s"' % escape(value, True)
        else:     value = ""

        if description: description = escape(description)
        else:           description = ""

        if self.ro_mode:
            if selected:
                out = "<option %s %s>%s</option>\n" % (value, selected, description)
        else:
            out = "<option %s %s>%s</option>\n" % (value, selected, description)

        self.toTarget(Text(out), bool.getTarget());

    def _renderRadioBool(self, bool):

        name        = bool.getName()
        pname       = bool.parent.getName()
        description = bool.getDescription()
        value       = bool.getValue()
        css_class   = []

        toCheck = self.stateStack[-1]

        checked = toCheck == value and "checked" or ""
        
        label = self._renderLabel(description, name, newline=False)

        if value:
            value = 'value="%s"' % escape(value, True)
        else:
            value = ""

        if   pname: oname = 'name="%s"' % escape(pname, True)
        elif  name: oname = 'name="%s"' % escape(name, True)
        else:       oname = ''

        if name: id = 'id="%s"' % escape(name, True)
        else:    id = ""
        
        if self.ro_mode: 
            disabled = "disabled"
            css_class.append(disabled)
        else: disabled = ""

        out = '<input type="radio" class="%s" %s %s %s %s %s %s>%s\n' % (
            " ".join(css_class), disabled, oname, id, value, checked, label, SET_MODIFICATION)

        self.toTarget(Item(out), bool.getTarget())

    def _renderPlainBool(self, bool):

        name        = bool.getName()
        description = bool.getDescription()
        css_class   = []

        data = self.formdata.getData(name)
        if data == "1": 
            value = 'value="%s"' % escape(data, True)
            checked = "checked"
        else:
            value, checked = "", ""

        label = self._renderLabel(description, name)

        if name: 
            name = escape(name, True)
            name = 'name="%s" id="%s"' % (name, name)
        else:
            name = ''

        if self.ro_mode: 
            disabled = "disabled"
            css_class.append('readonly')
        else: disabled = ""

        out = '<input type="checkbox" class="%s" %s %s %s %s %s>%s' % (
            " ".join(css_class), disabled, name, value, checked, SET_MODIFICATION, label)

        self.toTarget(Item(out), bool.getTarget())

    def _renderLabel(self, description, name, help=None, newline=True):
        error       = self.formdata.getError(name)
        if help: helpmsg = help
        else:    helpmsg = ""

        if newline: newline = "<br>"
        else:       newline = ""

        if description:
            if name:
                if error:
                    return '<label for="%s" class="error_font"><a name="f_%s">%s</a> %s</label>%s\n' % ( 
                    escape(name, True), escape(name, True), escape(description), helpmsg, newline)
                else:
                    return '<label for="%s"><a name="f_%s">%s</a> %s</label>%s\n' % ( 
                    escape(name, True), escape(name, True), escape(description), helpmsg, newline)
                    
            return "%s " % escape(description)
        return ""

    def _renderTextArea(self, text):

        description = text.getDescription()
        name        = text.getName()
        rows        = text.getRows()
        cols        = text.getCols()
        
        help        = self.formdata.getHelp(name)
        data        = self.formdata.getData(name)
        error       = self.formdata.getError(name)
        css_class   = []

        if data: value = escape(data, True)
        else:    value = ""

        if rows: rows = 'rows="%s"' % escape(rows, True)
        else:    rows = ""

        if cols: cols = 'cols="%s"' % escape(cols, True)
        else:    cols = ""

        out = [ self._renderLabel(description, name, help=help) ]

        if name: 
            name = escape(name, True)
            name = 'name="%s" id="%s"' % (name, name)
        else:
            name = ""

        if error:
            css_class.append('error')    

        if not self.ro_mode: 
            out.append('<textarea class="%s" %s %s %s %s>%s</textarea><br>\n' % (
            " ".join(css_class), name, rows, cols, SET_MODIFICATION, value))
        else:
            #disabled = "disabled"
            #css_class.append('readonly')
            value = value.replace('\r\n', '<br>')
            value = value.replace('\n', '<br>')
            value = value.replace('\r', '<br>')
            out.append('<p class="readonly">%s</p>' % F.NA(value))

        self.toTarget(Item("".join(out)), text.getTarget())

    def _renderDate(self, date):

        description = date.getDescription()
        name        = date.getName()
        help        = self.formdata.getHelp(name)

        data        = self.formdata.getData(name)
        error       = self.formdata.getError(name)
        css_class   = []

        if data: value = 'value="%s"' % escape(data, True)
        else:    value = ""
        
        out = [ self._renderLabel(description, name, help=help) ]

        if name:
            name = escape(name, True)
            name = 'name="%s" id="%s"' % (name, name)
        else:
            name = ""

        if self.ro_mode: 
            disabled = "readonly"
            css_class.append('readonly')
        else: disabled = ""

        if error:
            css_class.append('error_box')

        out.append( '<input type="text" size="10" maxlength="10" class="%s" %s %s %s %s><br>\n' % (
            " ".join(css_class), disabled, name, value, SET_MODIFICATION))

        self.toTarget(Item("".join(out)), date.getTarget());

    def _renderText(self, text):

        description = text.getDescription()
        size        = text.getSize()
        length      = text.getMaxLength()
        name        = text.getName()
        help        = self.formdata.getHelp(name)
        data        = self.formdata.getData(name)
        error       = self.formdata.getError(name)
        css_class   = ['field']

        if data: value = 'value="%s"' % escape(data, True)
        else:    value = ""

        if size:   size = 'size="%s"' % escape(size, True)
        else:      size = ""

        if length: length = 'maxlength="%s"' % escape(length, True)
        else:      length = ""

        if self.ro_mode: 
            css_class.append('readonly')
            disabled = "readonly"
        else: disabled = ""

        out = [ self._renderLabel(description, name, help=help) ]

        if name:

            name = escape(name, True)
            name = 'name="%s" id="%s"' % (name, name)
        else:
            name = ""

        if error:
            css_class.append('error_box')

        out.append('<input type="text" class="%s" %s %s %s %s %s %s><br>\n' % (
            " ".join(css_class), disabled, size, length, name, value, SET_MODIFICATION))

        self.toTarget(Item("".join(out)), text.getTarget())

    def _renderIntLeaf(self, integer):

        description = integer.getDescription()
        name        = integer.getName()
        minV        = integer.getMinValue()
        maxV        = integer.getMaxValue()
        help        = self.formdata.getHelp(name)
        data        = self.formdata.getData(name)
        error       = self.formdata.getError(name)
        css_class   = ['intfield']
        
        if data: value = 'value="%s"' % escape(data, True)
        else:    value = ""

        try:
            minV, maxV = int(minV), int(maxV)
            highest = max(abs(maxV), abs(minV))
            size = digits(highest)
            if minV < 0 or maxV < 0: size += 1
            size = 'size="%d" maxlength="%d"' % (size, size)
        except ValueError:
            size = ""

        out = [ self._renderLabel(description, name, help=help) ]

        if name:
            name = escape(name, True)
            id   = 'id="%s"'   % name
            name = 'name="%s"' % name
        else:
            id, name = "", ""

        if self.ro_mode: 
            disabled = "readonly"
            css_class.append('readonly')
        else: disabled = ""

        if error:
            css_class.append('error_box')
            
        out.append('<input type="text" class="%s" %s %s %s %s %s><br>\n' % (
            " ".join(css_class), disabled, size, name, value, SET_MODIFICATION))

        self.toTarget(Item("".join(out)), integer.getTarget())

    def _renderPage(self, page):
        out = self._renderRecursive(page)
        self.toTarget(Text(out), page.getTarget())

    def _renderBool(self, bool):

        parent = bool.parent

        if isinstance(parent,  data.ChoiceNode):
            self._renderChoiceBool(bool)

        elif isinstance(parent, data.RadioNode):
            self._renderRadioBool(bool)

        else:
            self._renderPlainBool(bool)

    def _renderExternalChoiceList(self, choiceList):
        children = choiceList.getChildren()
        if children:
            for child in children:
                if isinstance(child, data.BoolLeaf):
                    self._renderBool(child)

    def _renderChild(self, child):

        if isinstance(child, Node):
            if isinstance(child, data.GroupNode):
                self._renderGroup(child)
            elif isinstance(child, data.ChoiceNode):
                self._renderChoice(child)
            elif isinstance(child, data.RadioNode):
                self._renderRadio(child)
            elif isinstance(child, data.PageNode):
                self._renderPage(child)
            else:
                self.toTarget(Text(self._renderRecursive(child)), child.getTarget())

        elif isinstance(child, data.InfoLeaf):
            self._renderInfo(child)
        elif isinstance(child, data.BoolLeaf):
            self._renderBool(child)
        elif isinstance(child, data.TextLeaf):
            self._renderText(child)
        elif isinstance(child, data.TextAreaLeaf):
            self._renderTextArea(child)
        elif isinstance(child, data.IntLeaf):
            self._renderIntLeaf(child)
        elif isinstance(child, data.DateLeaf):
            self._renderDate(child)
        elif isinstance(child, data.ExternalChoiceListLeaf):
            self._renderExternalChoiceList(child)
        
    def _renderRecursive(self, node, doBreak=True):
        containers = node.getContainers()

        containers = containers and [Container(c.strip()) for c in containers.split(",")] or []
        containers.append(Container(None))

        self.containersStack.append(containers)

        for child in node.children:
            self._renderChild(child)

        out = []
        for c in self.containersStack.pop():
            txt = c.render(doBreak)
            if txt:
                out.append(txt)
                #if doBreak: out.append('<br class="newline">\n')
        return "".join(out)


# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf-8:
