# -*- coding: utf-8 -*-
#
# Copyright 2007, 2008 Intevation GmbH, Germany, <info@intevation.de>
# 
# This file is part of mpuls WASKA (CoMPUter-based case fiLeS - 
# Web-Anwendungs-Server fuer Kompetenzagenturen).
# 
# mpuls WASKA is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
# 
# mpuls WASKA is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
# 
# You should have received a copy of the GNU Affero General Public
# License along with mpuls WASKA. If not, see <http://www.gnu.org/licenses/>.
# 
# mpuls WASKA has been developed on behalf of the 
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and 
# European Social Fund resources.
#
# Authors:
# Sascha L. Teichmann <teichmann@intevation.de>
#

from types import StringTypes

import re
import sys
import xml.utils.iso8601 as iso8601
from   time import mktime


from pylons import g

import waskaweb.lib.helpers as h

from exportselection import RG_UNTERSTUETZUNG, SelectorFactory, CasePartsSelection, RG_ALL

FETCH_ALL_IDS = \
"""SELECT id FROM master_tbl_view"""

LOAD_LOGBOOK = """
SELECT l.uuid_id AS uuid_id, datum, b.login AS login, art, kurz_notiz, notiz, dauer
FROM ka_logbuch_tbl_view l LEFT JOIN ka_benutzer_tbl_view b
ON l.bearbeiter = b.id
WHERE l.master_id = %(id)s
ORDER BY datum
"""

LOAD_DOCUMENTS = """
SELECT uuid_id, groesse, mime, name, encode(daten, 'base64') AS daten
FROM ka_fall_dokumente_tbl_view
WHERE master_id = %(id)s
"""

BLACK_LIST = set(['id', 'master_id', 'bearbeiter_id'])

USER_LOGIN = re.compile(r"(?:[^_]*)_(?:[^_]*)_(.+)")

def login(a):
    b = USER_LOGIN.findall(a)
    return b and b[0] or a

def cdataWrap(value):
    return u"<![CDATA[%s]]>" % value.replace(u"]]>", u"]]><!CDATA[]]]]><![CDATA[>]]><[!CDATA[")

def escape(value, tagname):
    if tagname == 'uuid_id':
        return "%s" % unicode(str(value), "utf-8")
    if type(value) in StringTypes:
        return cdataWrap(unicode(str(value), "utf-8"))
    if value is None:
        return ""
    return "%s" % value


class LogbookEntry:

    def __init__(self, row):
        if row[0] is None: self.uuid_id = None
        else:              self.uuid_id = unicode(str(row[0]), 'UTF-8')

        self.datum = row[1]

        if row[2] is None: self.login = None
        else:              self.login = unicode(str(row[2]), 'UTF-8')

        self.art = row[3]

        if row[4] is None: self.kurz_notiz = None
        else:              self.kurz_notiz = unicode(str(row[4]), 'UTF-8')

        if row[5] is None: self.notiz = None
        else:              self.notiz = unicode(str(row[5]), 'UTF-8')

        self.dauer = row[6]

    def dump(self, out):
        out.write(u"<tagebuch-eintrag>")
        if not self.uuid_id is None:
            out.write(u"<uuid_id>%s</uuid_id>" % self.uuid_id)
        if not self.datum is None:
            out.write(u"<tagebuch-datum>%s</tagebuch-datum>" % 
                iso8601.tostring(mktime(self.datum.timetuple()) + 1e-6*self.datum.microsecond))
        if not self.login is None:
            out.write(u"<tagebuch-bearbeiter>%s</tagebuch-bearbeiter>" % login(self.login))
        if not self.art is None:
            out.write(u"<tagebuch-art>%d</tagebuch-art>" % self.art)
        if not self.kurz_notiz is None:
            out.write(u"<tagebuch-sonstiges>%s</tagebuch-sonstiges>" % cdataWrap(self.kurz_notiz))
        if not self.notiz is None:
            out.write(u"<tagebuch-notiz>%s</tagebuch-notiz>" % cdataWrap(self.notiz))
        if not self.dauer is None:
            out.write(u"<tagebuch-dauer>%d</tagebuch-dauer>" % 
                h.timedelta_in_minutes(self.dauer))
        out.write(u"</tagebuch-eintrag>")

class Logbook:

    def __init__(self):
        self.entries = []

    def addEntry(self, row):
        self.entries.append(LogbookEntry(row))

    def dump(self, out):
        if self.entries:
            out.write(u'<tagebuch>');
            for entry in self.entries:
                entry.dump(out)
            out.write(u'</tagebuch>');

class ExportCase:

    def __init__(self, master_id, selector):
        self.master_id = int(master_id)
        self.columns   = None
        self.rgs       = []
        self.selector  = selector
        self.logbook   = None
        self.blacklist = BLACK_LIST.copy()
        if selector.anonymize:
            self.blacklist.add('uuid_id')

    def dumpColums(self, columns):
        out = []
        for tagname, value in columns:
            if not value is None and tagname not in self.blacklist:
                out.append("<%s>%s</%s>" % (tagname, escape(value, tagname), tagname))
        return out

    def fetchData(self, cur):
        self.fetchMasterTable(cur)
        self.fetchRepeatGroups(cur)
        self.fetchLogbook(cur)

    def fetchMasterTable(self, cur):
        selector = self.selector
        stmnt = selector.getSelectStatementByView('master_tbl')
        if not stmnt:
            print >> sys.stderr, "No SELECT statement for master_tbl found"
            return
        cur.execute(stmnt, { 'id': self.master_id })
        row = cur.fetchone()
        if not row: raise Exception("No dataset for id %d" % self.master_id)
        columns = row._index.items() # XXX: hackish!
        columns.sort(lambda a, b: cmp(a[1], b[1]))
        columns = [(a[0], row[a[1]]) for a in columns]
        self.columns = columns

    def fetchRepeatGroups(self, cur):
        selector = self.selector
        id_dict = { 'id': self.master_id }
        for view in RG_ALL:
            stmnt = selector.getSelectStatementByView(view)
            if stmnt is None: continue
            cur.execute(stmnt, id_dict)
            rs = []
            while True:
                row = cur.fetchone()
                if not row: break
                cs = row._index.items() # XXX: hackish!
                cs.sort(lambda a, b: cmp(a[1], b[1]))
                cs = [(a[0], row[a[1]]) for a in cs]
                rs.append(cs)
            if rs:
                self.rgs.append((view, rs))

    def fetchLogbook(self, cur):
        if not self.selector.exportLogbook:
            return
        cur.execute(LOAD_LOGBOOK, {'id': self.master_id})
        logbook = Logbook()
        while True:
            row = cur.fetchone()
            if not row: break
            logbook.addEntry(row)
        self.logbook = logbook


    def dumpMasterTable(self, out):
        fields = self.dumpColums(self.columns)
        if fields:
            out.write("<master>%s</master>" % "".join(fields))

    def dumpRepeatGroups(self, out):
        for view, rgs in self.rgs:
            for r in rgs:
                fields = self.dumpColums(r)
                if fields:
                    out.write("<%s>%s</%s>" % (view, "".join(fields), view))

    def dumpLogbook(self, out):
        if self.logbook:
            self.logbook.dump(out)

    def dumpDocuments(self, out, cur):
        if not self.selector.exportDocuments:
            return
        # Not using OO aproach because loading documents first
        # would fill up memory
        first = True
        cur.execute(LOAD_DOCUMENTS, {'id': self.master_id})
        while True:
            row = cur.fetchone()
            if not row: break
            if first:
                first = False
                out.write("<anlagen>")
            out.write("<anlage>")
            if not row[0] is None:
                out.write("<uuid_id>%s</uuid_id>" % row[0])
            if not row[1] is None:
                out.write("<anlage-groesse>%d</anlage-groesse>" % row[1])
            if not row[2] is None:
                out.write("<anlage-mime>%s</anlage-mime>" % row[2])
            if not row[4] is None:
                out.write(u"<anlage-name>%s</anlage-name>" % cdataWrap(unicode(str(row[3]), 'UTF-8')))
            if not row[3] is None:
                out.write("<anlage-daten>")
                out.write(row[4])
                out.write("</anlage-daten>")
            out.write("</anlage>")

        if not first: out.write("</anlagen>")

    def dumpData(self, out, cur):
        out.write("<case>")
        self.dumpMasterTable(out)
        self.dumpRepeatGroups(out)
        self.dumpLogbook(out)
        self.dumpDocuments(out, cur)
        out.write("</case>")

def exportAsXML(cur, out, ids=None, case_parts=None, anonymize=False):
    if ids is None:
        cur.execute(FETCH_ALL_IDS)
        ids = []
        while True:
            row = cur.fetchone()
            ids.append(int(row[0]))

    if case_parts is None:
        case_parts = CasePartsSelection()

    selectFactory = SelectorFactory(case_parts, anonymize)

    selector = selectFactory.createSelector()

    out.write('<?xml version="1.0" encoding="UTF-8"?>\n')
    out.write("<cases>")
    for mid in ids:
        case = ExportCase(mid, selector)
        case.fetchData(cur)
        case.dumpData(out, cur)
    out.write("</cases>")

# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8:
